@@ -1,50 +1,30 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of Qt for Python.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial
 
-#include <QCoreApplication>
-#include <QLibrary>
-#include <QtCore/QFile>
-#include <QtCore/QDir>
-#include <QtCore/QVariant>
-#include <iostream>
-#include <apiextractor.h>
-#include <apiextractorresult.h>
-#include <fileout.h>
-#include <reporthandler.h>
-#include <typedatabase.h>
-#include <messages.h>
-#include "generator.h"
 #include "shibokenconfig.h"
 #include "cppgenerator.h"
+#include "generator.h"
 #include "headergenerator.h"
 #include "qtdocgenerator.h"
 
+#include <apiextractor.h>
+#include <apiextractorresult.h>
+#include <fileout.h>
+#include <messages.h>
+#include <reporthandler.h>
+#include <typedatabase.h>
+
+#include <QtCore/QDir>
+#include <QtCore/QFile>
+#include <QtCore/QLibrary>
+#include <QtCore/QVariant>
+
+#include "qtcompat.h"
+
 #include <exception>
+#include <iostream>
+
+using namespace Qt::StringLiterals;
 
 static const QChar clangOptionsSplitter = u',';
 static const QChar keywordsSplitter = u',';
@@ -54,6 +34,9 @@ static const QChar apiVersionSplitter = u'|';
 static inline QString keywordsOption() { return QStringLiteral("keywords"); }
 static inline QString clangOptionOption() { return QStringLiteral("clang-option"); }
 static inline QString clangOptionsOption() { return QStringLiteral("clang-options"); }
+static inline QString compilerOption() { return QStringLiteral("compiler"); }
+static inline QString compilerPathOption() { return QStringLiteral("compiler-path"); }
+static inline QString platformOption() { return QStringLiteral("platform"); }
 static inline QString apiVersionOption() { return QStringLiteral("api-version"); }
 static inline QString dropTypeEntriesOption() { return QStringLiteral("drop-type-entries"); }
 static inline QString languageLevelOption() { return QStringLiteral("language-level"); }
@@ -66,8 +49,10 @@ static inline QString diffOption() { return QStringLiteral("diff"); }
 static inline QString useGlobalHeaderOption() { return QStringLiteral("use-global-header"); }
 static inline QString dryrunOption() { return QStringLiteral("dry-run"); }
 static inline QString skipDeprecatedOption() { return QStringLiteral("skip-deprecated"); }
+static inline QString printBuiltinTypesOption() { return QStringLiteral("print-builtin-types"); }
 
 static const char helpHint[] = "Note: use --help or -h for more information.\n";
+static const char appName[] = "shiboken";
 
 using OptionDescriptions = Generator::OptionDescriptions;
 
@@ -86,6 +71,8 @@ struct CommandLineArguments
         addToOptionsList(option, pathListValue, QDir::listSeparator());
     }
 
+    bool addCommonOption(const QString &option, const QString &value);
+
     QVariantMap options; // string,stringlist for path lists, etc.
     QStringList positionalArguments;
 };
@@ -124,18 +111,42 @@ void CommandLineArguments::addToOptionsList(const QString &option,
     addToOptionsList(option, newValues);
 }
 
+// Add options common to project file and command line
+bool CommandLineArguments::addCommonOption(const QString &option,
+                                           const QString &value)
+{
+    bool result = true;
+    if (option == compilerOption() || option == compilerPathOption()
+        || option == platformOption()) {
+        options.insert(option, value);
+    } else if (option == clangOptionOption()) {
+        options.insert(option, QStringList(value));
+    } else if (option == clangOptionsOption()) {
+        addToOptionsList(option, value, clangOptionsSplitter);
+    } else if (option == apiVersionOption()) {
+        addToOptionsList(option, value, apiVersionSplitter);
+    } else if (option == keywordsOption()) {
+        addToOptionsList(option, value, keywordsSplitter);
+    } else if (option == dropTypeEntriesOption()) {
+        addToOptionsList(option, value, dropTypeEntriesSplitter);
+    } else {
+        result = false;
+    }
+    return result;
+}
+
 static void printOptions(QTextStream &s, const OptionDescriptions &options)
 {
     s.setFieldAlignment(QTextStream::AlignLeft);
     for (const auto &od : options) {
-        if (!od.first.startsWith(QLatin1Char('-')))
+        if (!od.first.startsWith(u'-'))
             s << "--";
         s << od.first;
         if (od.second.isEmpty()) {
             s << ", ";
         } else {
             s << Qt::endl;
-            const auto lines = QStringView{od.second}.split(QLatin1Char('\n'));
+            const auto lines = QStringView{od.second}.split(u'\n');
             for (const auto &line : lines)
                 s << "        " << line << Qt::endl;
             s << Qt::endl;
@@ -143,84 +154,71 @@ static void printOptions(QTextStream &s, const OptionDescriptions &options)
     }
 }
 
+// Return the file command line option matching a project file keyword
+static QString projectFileKeywordToCommandLineOption(const QString &p)
+{
+    if (p == u"include-path")
+        return includePathOption(); // "include-paths", ...
+    if (p == u"framework-include-path")
+        return frameworkIncludePathOption();
+    if (p == u"typesystem-path")
+        return typesystemPathOption();
+    if (p == u"system-include-paths")
+        return systemIncludePathOption();
+    return {};
+}
+
+static void processProjectFileLine(const QByteArray &line, CommandLineArguments &args)
+{
+    if (line.isEmpty())
+        return;
+    const QString lineS = QString::fromUtf8(line);
+    const auto split = line.indexOf(u'=');
+    if (split < 0) {
+        args.options.insert(lineS, QString{});
+        return;
+    }
+
+    const QString key = lineS.left(split).trimmed();
+    const QString value = lineS.mid(split + 1).trimmed();
+    const QString fileOption = projectFileKeywordToCommandLineOption(key);
+    if (fileOption.isEmpty()) {
+        if (key == u"header-file") {
+            args.positionalArguments.prepend(value);
+        } else if (key == u"typesystem-file") {
+            args.positionalArguments.append(value);
+        } else {
+            args.options.insert(key, value);
+        }
+    } else {
+        // Add single line value to the path list
+        args.addToOptionsList(fileOption, QDir::toNativeSeparators(value));
+    }
+}
+
 static std::optional<CommandLineArguments>
-    processProjectFile(const QString &appName, QFile &projectFile)
+    processProjectFile(const char *appName, QFile &projectFile)
 {
     QByteArray line = projectFile.readLine().trimmed();
     if (line.isEmpty() || line != "[generator-project]") {
-        std::cerr << qPrintable(appName) << ": first line of project file \""
+        std::cerr << appName << ": first line of project file \""
             << qPrintable(projectFile.fileName())
             << "\" must be the string \"[generator-project]\"\n";
         return {};
     }
 
     CommandLineArguments args;
-
-    while (!projectFile.atEnd()) {
-        line = projectFile.readLine().trimmed();
-        if (line.isEmpty())
-            continue;
-
-        int split = line.indexOf('=');
-        QByteArray key;
-        QString value;
-        if (split > 0) {
-            key = line.left(split).trimmed();
-            value = QString::fromUtf8(line.mid(split + 1).trimmed());
-        } else {
-            key = line;
-        }
-
-        if (key == "include-path") {
-            args.addToOptionsList(includePathOption(),
-                                  QDir::toNativeSeparators(value));
-        } else if (key == "framework-include-path") {
-            args.addToOptionsList(frameworkIncludePathOption(),
-                                  QDir::toNativeSeparators(value));
-        } else if (key == "system-include-paths") {
-            args.addToOptionsList(systemIncludePathOption(),
-                                  QDir::toNativeSeparators(value));
-        } else if (key == "typesystem-path") {
-            args.addToOptionsList(typesystemPathOption(),
-                                  QDir::toNativeSeparators(value));
-        } else if (key == "language-level") {
-            args.options.insert(languageLevelOption(), value);
-        } else if (key == "clang-option") {
-            args.addToOptionsList(clangOptionsOption(), value);
-        } else if (key == "clang-options") {
-            args.addToOptionsList(clangOptionsOption(),
-                                  value, clangOptionsSplitter);
-        } else if (key == "api-version") {
-            args.addToOptionsList(apiVersionOption(),
-                                  value, apiVersionSplitter);
-        } else if (key == "keywords") {
-            args.addToOptionsList(keywordsOption(),
-                                  value, keywordsSplitter);
-        } else if (key == "drop-type-entries") {
-            args.addToOptionsList(dropTypeEntriesOption(),
-                                  value, dropTypeEntriesSplitter);
-        } else if (key == "header-file") {
-            args.positionalArguments.prepend(value);
-        } else if (key == "typesystem-file") {
-            args.positionalArguments.append(value);
-        } else {
-            args.options.insert(QString::fromUtf8(key), value);
-        }
-    }
-
+    while (!projectFile.atEnd())
+        processProjectFileLine(projectFile.readLine().trimmed(), args);
     return args;
 }
 
-static std::optional<CommandLineArguments> getProjectFileArguments()
+static std::optional<CommandLineArguments> getProjectFileArguments(const QStringList &arguments)
 {
-    QStringList arguments = QCoreApplication::arguments();
-    QString appName = arguments.constFirst();
-    arguments.removeFirst();
-
     QString projectFileName;
-    for (const QString &arg : qAsConst(arguments)) {
-        if (arg.startsWith(QLatin1String("--project-file"))) {
-            int split = arg.indexOf(QLatin1Char('='));
+    for (const QString &arg : std::as_const(arguments)) {
+        if (arg.startsWith(u"--project-file")) {
+            int split = arg.indexOf(u'=');
             if (split > 0)
                 projectFileName = arg.mid(split + 1).trimmed();
             break;
@@ -231,14 +229,14 @@ static std::optional<CommandLineArguments> getProjectFileArguments()
         return CommandLineArguments{};
 
     if (!QFile::exists(projectFileName)) {
-        std::cerr << qPrintable(appName) << ": Project file \""
+        std::cerr << appName << ": Project file \""
             << qPrintable(projectFileName) << "\" not found.\n";
         return {};
     }
 
     QFile projectFile(projectFileName);
     if (!projectFile.open(QIODevice::ReadOnly)) {
-        std::cerr << qPrintable(appName) << ": Cannot open project file \""
+        std::cerr << appName << ": Cannot open project file \""
             << qPrintable(projectFileName) << "\" : " << qPrintable(projectFile.errorString())
             << '\n';
         return {};
@@ -248,46 +246,38 @@ static std::optional<CommandLineArguments> getProjectFileArguments()
 
 static void getCommandLineArg(QString arg, int &argNum, CommandLineArguments &args)
 {
-    if (arg.startsWith(QLatin1String("--"))) {
+    if (arg.startsWith(u"--")) {
         arg.remove(0, 2);
-        const int split = arg.indexOf(QLatin1Char('='));
+        const auto split = arg.indexOf(u'=');
         if (split < 0) {
             args.options.insert(arg, QString());
             return;
         }
         const QString option = arg.left(split);
         const QString value = arg.mid(split + 1).trimmed();
-        if (option == includePathOption() || option == frameworkIncludePathOption()
-            || option == systemIncludePathOption() || option == typesystemPathOption()) {
+        if (args.addCommonOption(option, value)) {
+        } else if (option == includePathOption() || option == frameworkIncludePathOption()
+                   || option == systemIncludePathOption() || option == typesystemPathOption()) {
+            // Add platform path-separator separated list value to path list
             args.addToOptionsPathList(option, value);
-        } else if (option == apiVersionOption()) {
-            args.addToOptionsList(apiVersionOption(), value, apiVersionSplitter);
-        } else if (option == dropTypeEntriesOption()) {
-            args.addToOptionsList(dropTypeEntriesOption(), value, dropTypeEntriesSplitter);
-        } else if (option == clangOptionOption()) {
-            args.addToOptionsList(clangOptionsOption(), value);
-        } else if (option == clangOptionsOption()) {
-            args.addToOptionsList(clangOptionsOption(), value, clangOptionsSplitter);
-        } else if (option == keywordsOption()) {
-            args.addToOptionsList(keywordsOption(), value, keywordsSplitter);
         } else {
             args.options.insert(option, value);
         }
         return;
     }
-    if (arg.startsWith(QLatin1Char('-'))) {
+    if (arg.startsWith(u'-')) {
         arg.remove(0, 1);
-        if (arg.startsWith(QLatin1Char('I'))) // Shorthand path arguments -I/usr/include...
+        if (arg.startsWith(u'I')) // Shorthand path arguments -I/usr/include...
             args.addToOptionsPathList(includePathOption(), arg.mid(1));
-        else if (arg.startsWith(QLatin1Char('F')))
+        else if (arg.startsWith(u'F'))
             args.addToOptionsPathList(frameworkIncludePathOption(), arg.mid(1));
-        else if (arg.startsWith(QLatin1String("isystem")))
+        else if (arg.startsWith(u"isystem"))
             args.addToOptionsPathList(systemIncludePathOption(), arg.mid(7));
-        else if (arg.startsWith(QLatin1Char('T')))
+        else if (arg.startsWith(u'T'))
             args.addToOptionsPathList(typesystemPathOption(), arg.mid(1));
-        else if (arg == QLatin1String("h"))
+        else if (arg == u"h")
             args.options.insert(helpOption(), QString());
-        else if (arg.startsWith(QLatin1String("std=")))
+        else if (arg.startsWith(u"std="))
             args.options.insert(languageLevelOption(), arg.mid(4));
         else
             args.options.insert(arg, QString());
@@ -300,12 +290,11 @@ static void getCommandLineArg(QString arg, int &argNum, CommandLineArguments &ar
     ++argNum;
 }
 
-static void getCommandLineArgs(CommandLineArguments &args)
+static void getCommandLineArgs(CommandLineArguments &args, const QStringList &arguments)
 {
-    const QStringList arguments = QCoreApplication::arguments();
     int argNum = 0;
-    for (int i = 1, size = arguments.size(); i < size; ++i)
-        getCommandLineArg(arguments.at(i).trimmed(), argNum, args);
+    for (const QString &argument : arguments)
+        getCommandLineArg(argument.trimmed(), argNum, args);
 }
 
 static inline Generators docGenerators()
@@ -326,9 +315,9 @@ static inline Generators shibokenGenerators()
 
 static inline QString languageLevelDescription()
 {
-    return QLatin1String("C++ Language level (c++11..c++17, default=")
-        + QLatin1String(clang::languageLevelOption(clang::emulatedCompilerLanguageLevel()))
-        + QLatin1Char(')');
+    return u"C++ Language level (c++11..c++17, default="_s
+        + QLatin1StringView(clang::languageLevelOption(clang::emulatedCompilerLanguageLevel()))
+        + u')';
 }
 
 void printUsage()
@@ -342,57 +331,65 @@ void printUsage()
     QTextStream(&pathSyntax) << "<path>[" << pathSplitter << "<path>"
         << pathSplitter << "...]";
     OptionDescriptions generalOptions = {
-        {QLatin1String("api-version=<\"package mask\">,<\"version\">"),
-         QLatin1String("Specify the supported api version used to generate the bindings")},
-        {QLatin1String("debug-level=[sparse|medium|full]"),
-         QLatin1String("Set the debug level")},
-        {QLatin1String("documentation-only"),
-         QLatin1String("Do not generates any code, just the documentation")},
-        {QLatin1String("drop-type-entries=\"<TypeEntry0>[;TypeEntry1;...]\""),
-         QLatin1String("Semicolon separated list of type system entries (classes, namespaces,\n"
-                       "global functions and enums) to be dropped from generation.")},
+        {u"api-version=<\"package mask\">,<\"version\">"_s,
+         u"Specify the supported api version used to generate the bindings"_s},
+        {u"debug-level=[sparse|medium|full]"_s,
+         u"Set the debug level"_s},
+        {u"documentation-only"_s,
+         u"Do not generates any code, just the documentation"_s},
+        {u"drop-type-entries=\"<TypeEntry0>[;TypeEntry1;...]\""_s,
+         u"Semicolon separated list of type system entries (classes, namespaces,\n"
+          "global functions and enums) to be dropped from generation."_s},
         {keywordsOption() + QStringLiteral("=keyword1[,keyword2,...]"),
-         QLatin1String("A comma-separated list of keywords for conditional typesystem parsing")},
+         u"A comma-separated list of keywords for conditional typesystem parsing"_s},
         {clangOptionOption(),
-         QLatin1String("Option to be passed to clang")},
+         u"Option to be passed to clang"_s},
         {clangOptionsOption(),
-         QLatin1String("A comma-separated list of options to be passed to clang")},
-        {QLatin1String("-F<path>"), {} },
-        {QLatin1String("framework-include-paths=") + pathSyntax,
-         QLatin1String("Framework include paths used by the C++ parser")},
-        {QLatin1String("-isystem<path>"), {} },
-        {QLatin1String("system-include-paths=") + pathSyntax,
-         QLatin1String("System include paths used by the C++ parser")},
+         u"A comma-separated list of options to be passed to clang"_s},
+        {compilerOption() + u"=<type>"_s,
+         u"Emulated compiler type (g++, msvc, clang)"_s},
+        {platformOption() + u"=<name>"_s,
+         u"Emulated platform (windows, darwin, unix)"_s},
+        {compilerPathOption() + u"=<file>"_s,
+         u"Path to the compiler for determining builtin include paths"_s},
+        {u"-F<path>"_s, {} },
+        {u"framework-include-paths="_s + pathSyntax,
+         u"Framework include paths used by the C++ parser"_s},
+        {u"-isystem<path>"_s, {} },
+        {u"system-include-paths="_s + pathSyntax,
+         u"System include paths used by the C++ parser"_s},
         {useGlobalHeaderOption(),
-         QLatin1String("Use the global headers in generated code.")},
-        {QLatin1String("generator-set=<\"generator module\">"),
-         QLatin1String("generator-set to be used. e.g. qtdoc")},
+         u"Use the global headers in generated code."_s},
+        {u"generator-set=<\"generator module\">"_s,
+         u"generator-set to be used. e.g. qtdoc"_s},
         {skipDeprecatedOption(),
-         QLatin1String("Skip deprecated functions")},
-        {diffOption(), QLatin1String("Print a diff of wrapper files")},
-        {dryrunOption(), QLatin1String("Dry run, do not generate wrapper files")},
-        {QLatin1String("-h"), {} },
-        {helpOption(), QLatin1String("Display this help and exit")},
-        {QLatin1String("-I<path>"), {} },
-        {QLatin1String("include-paths=") + pathSyntax,
-        QLatin1String("Include paths used by the C++ parser")},
-        {languageLevelOption() + QLatin1String("=, -std=<level>"),
+         u"Skip deprecated functions"_s},
+        {diffOption(), u"Print a diff of wrapper files"_s},
+        {dryrunOption(), u"Dry run, do not generate wrapper files"_s},
+        {u"-h"_s, {} },
+        {helpOption(), u"Display this help and exit"_s},
+        {u"-I<path>"_s, {} },
+        {u"include-paths="_s + pathSyntax,
+        u"Include paths used by the C++ parser"_s},
+        {languageLevelOption() + u"=, -std=<level>"_s,
          languageLevelDescription()},
-        {QLatin1String("license-file=<license-file>"),
-         QLatin1String("File used for copyright headers of generated files")},
-        {QLatin1String("no-suppress-warnings"),
-         QLatin1String("Show all warnings")},
-        {QLatin1String("output-directory=<path>"),
-         QLatin1String("The directory where the generated files will be written")},
-        {QLatin1String("project-file=<file>"),
-         QLatin1String("text file containing a description of the binding project.\n"
-                       "Replaces and overrides command line arguments")},
-        {QLatin1String("silent"), QLatin1String("Avoid printing any message")},
-        {QLatin1String("-T<path>"), {} },
-        {QLatin1String("typesystem-paths=") + pathSyntax,
-         QLatin1String("Paths used when searching for typesystems")},
-        {QLatin1String("version"),
-         QLatin1String("Output version information and exit")}
+        {u"license-file=<license-file>"_s,
+         u"File used for copyright headers of generated files"_s},
+        {u"no-suppress-warnings"_s,
+         u"Show all warnings"_s},
+        {u"output-directory=<path>"_s,
+         u"The directory where the generated files will be written"_s},
+        {u"project-file=<file>"_s,
+         u"text file containing a description of the binding project.\n"
+          "Replaces and overrides command line arguments"_s},
+        {u"silent"_s, u"Avoid printing any message"_s},
+        {u"-T<path>"_s, {} },
+        {u"typesystem-paths="_s + pathSyntax,
+         u"Paths used when searching for typesystems"_s},
+        {printBuiltinTypesOption(),
+         u"Print information about builtin types"_s},
+        {u"version"_s,
+         u"Output version information and exit"_s}
     };
     printOptions(s, generalOptions);
 
@@ -408,15 +405,13 @@ void printUsage()
 
 static inline void printVerAndBanner()
 {
-    std::cout << "shiboken v" SHIBOKEN_VERSION << std::endl;
+    std::cout << appName << " v" << SHIBOKEN_VERSION << std::endl;
     std::cout << "Copyright (C) 2016 The Qt Company Ltd." << std::endl;
 }
 
-static inline void errorPrint(const QString &s)
+static inline void errorPrint(const QString &s, const QStringList &arguments)
 {
-    QStringList arguments = QCoreApplication::arguments();
-    arguments.pop_front();
-    std::cerr << "shiboken: " << qPrintable(s) << "\nCommand line:\n";
+    std::cerr << appName << ": " << qPrintable(s) << "\nCommand line:\n";
     for (const auto &argument : arguments)
         std::cerr << "    \"" << qPrintable(argument) << "\"\n";
 }
@@ -436,28 +431,27 @@ static void parseIncludePathOption(const QString &option, HeaderType headerType,
     }
 }
 
-int shibokenMain(int argc, char *argv[])
+int shibokenMain(const QStringList &argV)
 {
     // PYSIDE-757: Request a deterministic ordering of QHash in the code model
     // and type system.
-    qSetGlobalQHashSeed(0);
-    // needed by qxmlpatterns
-    QCoreApplication app(argc, argv);
+    QHashSeed::setDeterministicGlobalSeed();
+
     ReportHandler::install();
     if (ReportHandler::isDebug(ReportHandler::SparseDebug))
-        qCInfo(lcShiboken()).noquote().nospace() << QCoreApplication::arguments().join(QLatin1Char(' '));
+        qCInfo(lcShiboken()).noquote().nospace() << appName << ' ' << argV.join(u' ');
 
     // Store command arguments in a map
-    const auto projectFileArgumentsOptional = getProjectFileArguments();
+    const auto projectFileArgumentsOptional = getProjectFileArguments(argV);
     if (!projectFileArgumentsOptional.has_value())
         return EXIT_FAILURE;
 
     const CommandLineArguments projectFileArguments = projectFileArgumentsOptional.value();
     CommandLineArguments args = projectFileArguments;
-    getCommandLineArgs(args);
+    getCommandLineArgs(args, argV);
     Generators generators;
 
-    auto ait = args.options.find(QLatin1String("version"));
+    auto ait = args.options.find(u"version"_s);
     if (ait != args.options.end()) {
         args.options.erase(ait);
         printVerAndBanner();
@@ -465,29 +459,29 @@ int shibokenMain(int argc, char *argv[])
     }
 
     QString generatorSet;
-    ait = args.options.find(QLatin1String("generator-set"));
-    if (ait == args.options.end()) // Also check QLatin1String("generatorSet") command line argument for backward compatibility.
-        ait = args.options.find(QLatin1String("generatorSet"));
+    ait = args.options.find(u"generator-set"_s);
+    if (ait == args.options.end()) // Also check "generatorSet" command line argument for backward compatibility.
+        ait = args.options.find(u"generatorSet"_s);
     if (ait != args.options.end()) {
         generatorSet = ait.value().toString();
         args.options.erase(ait);
     }
 
     // Pre-defined generator sets.
-    if (generatorSet == QLatin1String("qtdoc")) {
+    if (generatorSet == u"qtdoc") {
         generators = docGenerators();
         if (generators.isEmpty()) {
-            errorPrint(QLatin1String("Doc strings extractions was not enabled in this shiboken build."));
+            errorPrint(u"Doc strings extractions was not enabled in this shiboken build."_s, argV);
             return EXIT_FAILURE;
         }
-    } else if (generatorSet.isEmpty() || generatorSet == QLatin1String("shiboken")) {
+    } else if (generatorSet.isEmpty() || generatorSet == u"shiboken") {
         generators = shibokenGenerators();
     } else {
-        errorPrint(QLatin1String("Unknown generator set, try \"shiboken\" or \"qtdoc\"."));
+        errorPrint(u"Unknown generator set, try \"shiboken\" or \"qtdoc\"."_s, argV);
         return EXIT_FAILURE;
     }
 
-    ait = args.options.find(QLatin1String("help"));
+    ait = args.options.find(u"help"_s);
     if (ait != args.options.end()) {
         args.options.erase(ait);
         printUsage();
@@ -513,7 +507,7 @@ int shibokenMain(int argc, char *argv[])
     }
 
     QString licenseComment;
-    ait = args.options.find(QLatin1String("license-file"));
+    ait = args.options.find(u"license-file"_s);
     if (ait != args.options.end()) {
         QFile licenseFile(ait.value().toString());
         args.options.erase(ait);
@@ -521,13 +515,13 @@ int shibokenMain(int argc, char *argv[])
             licenseComment = QString::fromUtf8(licenseFile.readAll());
         } else {
             errorPrint(QStringLiteral("Could not open the file \"%1\" containing the license heading: %2").
-                       arg(QDir::toNativeSeparators(licenseFile.fileName()), licenseFile.errorString()));
+                       arg(QDir::toNativeSeparators(licenseFile.fileName()), licenseFile.errorString()), argV);
             return EXIT_FAILURE;
         }
     }
 
-    QString outputDirectory = QLatin1String("out");
-    ait = args.options.find(QLatin1String("output-directory"));
+    QString outputDirectory = u"out"_s;
+    ait = args.options.find(u"output-directory"_s);
     if (ait != args.options.end()) {
         outputDirectory = ait.value().toString();
         args.options.erase(ait);
@@ -550,22 +544,22 @@ int shibokenMain(int argc, char *argv[])
         args.options.erase(ait);
     }
 
-    ait = args.options.find(QLatin1String("silent"));
+    ait = args.options.find(u"silent"_s);
     if (ait != args.options.end()) {
         extractor.setSilent(true);
         args.options.erase(ait);
     } else {
-        ait = args.options.find(QLatin1String("debug-level"));
+        ait = args.options.find(u"debug-level"_s);
         if (ait != args.options.end()) {
             const QString value = ait.value().toString();
             if (!ReportHandler::setDebugLevelFromArg(value)) {
-                errorPrint(QLatin1String("Invalid debug level: ") + value);
+                errorPrint(u"Invalid debug level: "_s + value, argV);
                 return EXIT_FAILURE;
             }
             args.options.erase(ait);
         }
     }
-    ait = args.options.find(QLatin1String("no-suppress-warnings"));
+    ait = args.options.find(u"no-suppress-warnings"_s);
     if (ait != args.options.end()) {
         args.options.erase(ait);
         extractor.setSuppressWarnings(false);
@@ -575,13 +569,13 @@ int shibokenMain(int argc, char *argv[])
         const QStringList &versions = ait.value().toStringList();
         args.options.erase(ait);
         for (const QString &fullVersion : versions) {
-            QStringList parts = fullVersion.split(QLatin1Char(','));
+            QStringList parts = fullVersion.split(u',');
             QString package;
             QString version;
-            package = parts.size() == 1 ? u"*"_qs : parts.constFirst();
+            package = parts.size() == 1 ? u"*"_s : parts.constFirst();
             version = parts.constLast();
             if (!extractor.setApiVersion(package, version)) {
-                errorPrint(msgInvalidVersion(package, version));
+                errorPrint(msgInvalidVersion(package, version), argV);
                 return EXIT_FAILURE;
             }
         }
@@ -611,6 +605,37 @@ int shibokenMain(int argc, char *argv[])
         args.options.erase(ait);
     }
 
+    ait = args.options.find(compilerOption());
+    if (ait != args.options.end()) {
+        const QString name = ait.value().toString();
+        if (!clang::setCompiler(name)) {
+            errorPrint(u"Invalid value \""_s + name + u"\" passed to --compiler"_s, argV);
+            return EXIT_FAILURE;
+        }
+        args.options.erase(ait);
+    }
+
+    ait = args.options.find(printBuiltinTypesOption());
+    const bool printBuiltinTypes = ait != args.options.end();
+    if (printBuiltinTypes)
+        args.options.erase(ait);
+
+    ait = args.options.find(compilerPathOption());
+    if (ait != args.options.end()) {
+        clang::setCompilerPath(ait.value().toString());
+        args.options.erase(ait);
+    }
+
+    ait = args.options.find(platformOption());
+    if (ait != args.options.end()) {
+        const QString name = ait.value().toString();
+        if (!clang::setPlatform(name)) {
+            errorPrint(u"Invalid value \""_s + name + u"\" passed to --platform"_s, argV);
+            return EXIT_FAILURE;
+        }
+        args.options.erase(ait);
+    }
+
     parseIncludePathOption(includePathOption(), HeaderType::Standard,
                            args, extractor);
     parseIncludePathOption(frameworkIncludePathOption(), HeaderType::Framework,
@@ -619,7 +644,8 @@ int shibokenMain(int argc, char *argv[])
                            args, extractor);
 
     if (args.positionalArguments.size() < 2) {
-        errorPrint(QLatin1String("Insufficient positional arguments, specify header-file and typesystem-file."));
+        errorPrint(u"Insufficient positional arguments, specify header-file and typesystem-file."_s,
+                  argV);
         std::cout << '\n';
         printUsage();
         return EXIT_FAILURE;
@@ -627,15 +653,15 @@ int shibokenMain(int argc, char *argv[])
 
     const QString typeSystemFileName = args.positionalArguments.takeLast();
     QString messagePrefix = QFileInfo(typeSystemFileName).baseName();
-    if (messagePrefix.startsWith(QLatin1String("typesystem_")))
+    if (messagePrefix.startsWith(u"typesystem_"))
         messagePrefix.remove(0, 11);
-    ReportHandler::setPrefix(QLatin1Char('(') + messagePrefix + QLatin1Char(')'));
+    ReportHandler::setPrefix(u'(' + messagePrefix + u')');
 
     QFileInfoList cppFileNames;
-    for (const QString &cppFileName : qAsConst(args.positionalArguments)) {
+    for (const QString &cppFileName : std::as_const(args.positionalArguments)) {
         const QFileInfo cppFileNameFi(cppFileName);
         if (!cppFileNameFi.isFile() && !cppFileNameFi.isSymLink()) {
-            errorPrint(QLatin1Char('"') + cppFileName + QLatin1String("\" does not exist."));
+            errorPrint(u'"' + cppFileName + u"\" does not exist."_s, argV);
             return EXIT_FAILURE;
         }
         cppFileNames.append(cppFileNameFi);
@@ -644,7 +670,7 @@ int shibokenMain(int argc, char *argv[])
     // Pass option to all generators (Cpp/Header generator have the same options)
     for (ait = args.options.begin(); ait != args.options.end(); ) {
         bool found = false;
-        for (const GeneratorPtr &generator : qAsConst(generators))
+        for (const GeneratorPtr &generator : std::as_const(generators))
             found |= generator->handleOption(ait.key(), ait.value().toString());
         if (found)
             ait = args.options.erase(ait);
@@ -669,14 +695,14 @@ int shibokenMain(int argc, char *argv[])
      * --project-file, also the arguments of each generator before
      * checking if there isn't any existing arguments in argsHandler.
      */
-    args.options.remove(QLatin1String("project-file"));
+    args.options.remove(u"project-file"_s);
     for (auto it = projectFileArguments.options.cbegin(), end = projectFileArguments.options.cend();
          it != end; ++it) {
         args.options.remove(it.key());
     }
 
     if (!args.options.isEmpty()) {
-        errorPrint(msgLeftOverArguments(args.options));
+        errorPrint(msgLeftOverArguments(args.options, argV), argV);
         std::cout << helpHint;
         return EXIT_FAILURE;
     }
@@ -697,7 +723,7 @@ int shibokenMain(int argc, char *argv[])
     const std::optional<ApiExtractorResult> apiOpt = extractor.run(apiExtractorFlags);
 
     if (!apiOpt.has_value()) {
-        errorPrint(QLatin1String("Error running ApiExtractor."));
+        errorPrint(u"Error running ApiExtractor."_s, argV);
         return EXIT_FAILURE;
     }
 
@@ -710,15 +736,18 @@ int shibokenMain(int argc, char *argv[])
             << "\n\nType datase:\n" << *TypeDatabase::instance();
     }
 
-    for (const GeneratorPtr &g : qAsConst(generators)) {
+    if (printBuiltinTypes)
+        TypeDatabase::instance()->formatBuiltinTypes(qInfo());
+
+    for (const GeneratorPtr &g : std::as_const(generators)) {
         g->setOutputDirectory(outputDirectory);
         g->setLicenseComment(licenseComment);
-        ReportHandler::startProgress(QByteArray("Running ") + g->name() + "...");
+        ReportHandler::startProgress("Ran "_ba + g->name() + '.');
         const bool ok = g->setup(apiOpt.value()) && g->generate();
         ReportHandler::endProgress();
          if (!ok) {
-             errorPrint(QLatin1String("Error running generator: ")
-                        + QLatin1String(g->name()) + QLatin1Char('.'));
+             errorPrint(u"Error running generator: "_s
+                        + QLatin1StringView(g->name()) + u'.', argV);
              return EXIT_FAILURE;
          }
     }
@@ -729,13 +758,34 @@ int shibokenMain(int argc, char *argv[])
     return EXIT_SUCCESS;
 }
 
+#ifndef Q_OS_WIN
+
+static inline QString argvToString(const char *arg)
+{
+    return QString::fromLocal8Bit(arg);
+}
+
 int main(int argc, char *argv[])
+#else
+
+static inline QString argvToString(const  wchar_t *arg)
+{
+    return QString::fromWCharArray(arg);
+}
+
+int wmain(int argc, wchar_t *argv[])
+#endif
 {
     int ex = EXIT_SUCCESS;
+
+    QStringList argV;
+    argV.reserve(argc - 1);
+    std::transform(argv + 1, argv + argc, std::back_inserter(argV), argvToString);
+
     try {
-        ex = shibokenMain(argc, argv);
+        ex = shibokenMain(argV);
     }  catch (const std::exception &e) {
-        std::cerr << e.what() << std::endl;
+        std::cerr << appName << " error: " << e.what() << std::endl;
         ex = EXIT_FAILURE;
     }
     return ex;
