@@ -1,5 +1,5 @@
 // Copyright (C) 2021 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial
 
 
 #ifdef HAVE_NUMPY
@@ -7,6 +7,7 @@
 #  include <numpy/arrayobject.h>
 #endif
 
+#include "helper.h"
 #include "sbknumpycheck.h"
 #include "sbkcpptonumpy.h"
 #include "sbknumpyview.h"
@@ -21,6 +22,7 @@ bool check(PyObject *pyIn)
 #ifdef HAVE_NUMPY
     return PyArray_Check(pyIn);
 #else
+    SBK_UNUSED(pyIn);
     return false;
 #endif
 }
