@@ -1,16 +1,17 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial
 
 #ifndef LISTUSER_H
 #define LISTUSER_H
 
-#include <list>
 #include "complex.h"
 #include "point.h"
 #include "pointf.h"
 
 #include "libsamplemacros.h"
 
+#include <list>
+
 class LIBSAMPLE_API ListUser
 {
 public:
@@ -23,9 +24,9 @@ public:
 
     ListUser();
     ListUser(const ListUser &other);
-    ListUser(ListUser &&other);
+    ListUser(ListUser &&other) noexcept;
     ListUser &operator=(const ListUser &other);
-    ListUser &operator=(ListUser &&other);
+    ListUser &operator=(ListUser &&other) noexcept;
     virtual ~ListUser();
 
     virtual std::list<int> createList();
@@ -36,13 +37,13 @@ public:
     double sumList(std::list<int> vallist);
     double sumList(std::list<double> vallist);
 
-    static ListOfSomething listOfPoints(const std::list<Point>& pointlist);
-    static ListOfSomething listOfPoints(const std::list<PointF>& pointlist);
+    static ListOfSomething listOfPoints(const std::list<Point> &pointlist);
+    static ListOfSomething listOfPoints(const std::list<PointF> &pointlist);
 
-    static void multiplyPointList(PointList& points, double multiplier);
+    static void multiplyPointList(PointList &points, double multiplier);
 
     inline void setList(std::list<int> lst) { m_lst = lst; }
-    inline std::list<int> getList() { return m_lst; }
+    inline std::list<int> getList() const { return m_lst; }
 
 private:
     std::list<int> m_lst;
