@@ -1,5 +1,5 @@
 # Copyright (C) 2022 The Qt Company Ltd.
-# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+# SPDX-License-Identifier: LicenseRef-Qt-Commercial
 
 """
 This script is used by Coin (coin_test_instructions.py specifically) to
@@ -82,10 +82,13 @@ def get_examples_dir():
 def package_prefix_names():
     # Note: shiboken6_generator is not needed for compile_using_nuitka,
     # but building modules with cmake needs it.
+    _shiboken_wheels = ["shiboken6", "shiboken6_generator"]
+    _old_pyside_wheels = ["PySide6"]
+    _new_pyside_wheels = ["PySide6_Essentials", "PySide6_Addons", "PySide6_M2M", "PySide6"]
     if NEW_WHEELS:
-        return ["shiboken6", "shiboken6_generator", "PySide6_Essentials", "PySide6_Addons", "PySide6"]
+        return _shiboken_wheels + _new_pyside_wheels
     else:
-        return ["shiboken6", "shiboken6_generator", "PySide6"]
+        return _shiboken_wheels + _old_pyside_wheels
 
 
 def clean_egg_info():
